#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <time.h>

#include "pqueue.h"

#define MAX 10

bool int_lt(void const * const a, void const * const b)
   {
   return  *((int *) a) < *((int *) b);
   }

bool int_gt(void const * const a, void const * const b)
   {
   return  *((int *) a) > *((int *) b);
   }

int main(int argc, char**argv)
   {
   struct pqueue * pq = pqueue_new(MAX, int_gt);
   if (NULL == pq)
      {
      perror("pqueue_new");
      exit(EXIT_FAILURE);
      }

   srand(time(NULL));

   // Fill the queue
   for (int i = 0; i < MAX; i++)
      {
      int * ip = malloc(sizeof(int));
      *ip = rand();
      if (NULL == ip)
	 {
	 perror("malloc");
	 exit(EXIT_FAILURE);
	 }

      if (NULL == pqueue_push(pq, ip))
	 {
	 perror("pqueue_push");
	 exit(EXIT_FAILURE);
	 }
      }

   printf("size is %zd\n", pqueue_size(pq));

   int * ip;
   printf("peeking...");
   ip = pqueue_peek(pq);
   if (NULL == ip)
      {
      perror("pqueue_peek");
      exit(EXIT_FAILURE);
      }
   printf("%d\n", *ip);

   printf("size is %zd\n", pqueue_size(pq));

   // Empty the queue
   while (ip = pqueue_pop(pq))
      {
      printf("%d\n", *ip);
      free(ip);
      }

   printf("size is %zd\n", pqueue_size(pq));

   pqueue_delete(pq);
   }
